%%###########à  TEST   ########################
%
%  The main predicates are:
%
%    *  test(+Lab)
%
%	Run the test with label Lab and print the output on the standard
%	output.
%
%	The output is a term of the form
%
%	  proof(Rule_name, Sequent, Proof_list)
%
%	representing a proof of Sequent or a term of the form
%
%	  ref(Rule_name, Sequent, Ref_list)
%
%      representing a refutation of Sequent (see the syntax in
%      prover.pl).
%
%    *  test(+Lab, -Result)
%
%	Run the test with label Lab and write the result (proof or
%	refutation) in a latex file. Result will be unified with the
%	term representing the output.
%
%       Latex files are placed in the directory defined test_dir/1
%       and have a name of the form proof_Lab.tex or ref_Lab.tex
%	according to the content.
%
%    *  test_input(+Lab, +Seq, +Pdf)
%
%       Define the input test with label Lab.
%
%	o Seq is the sequent to be proved (see the syntax in prover.pl)
%	o Pdf must pdf (compile the latex file containing the result)
%	  or nopdf (only generate tex file).
%
%    To run a test with label test_label  from command line:
%
%     swipl -q -f  test.pl -t 'test(test_label)'
%     swipl -q -f  test.pl -t 'test(test_label,Result)'
%
%    To run all the defined  tests:
%
%      swipl -q -f test.pl -t alltests
%

:- module(test, [
	         test/1,
		 test/2,
		 alltests/0
	      ]
	   ).


:- set_prolog_flag(toplevel_print_options, [max_depth(20)]).

:-use_module(prover).
:-use_module(util).

%% SETTING

test_dir(tests).  % set the directory for the output files



 %%% BASIC RULE TESTS

%%   box_box_l

%ref
test_input(bbl1, Seq,pdf) :-
       Gamma = [ boxp p1, boxp p2, diamc q1, diamc q2,  box a1, box a2 , box diam b1, box diam b2],
       Delta =  [g2],
       Seq = Gamma ==> Delta.

%proof
test_input(bbl2, Seq,pdf) :-
       Gamma  = [boxp p1, boxp ~b, diamc q1, diamc ~a ,  box a, box a1 , box diam b1, box diam b, g1 ],
       Delta = [g2],
       Seq = Gamma ==> Delta.


%proof
test_input(bbl3, Seq,pdf) :-
       Gamma =  [boxp p1,  diamc q1, diamc ~q2, box a1, box a2 , box diam b1, box diam b2, g1 ],
       Delta = [g1],
       Seq = Gamma ==> Delta.

%%  box_or_box_l

%ref
test_input(bob0, Seq,pdf) :-
       Gamma = [box ( c1 v box d1), box ( c2 v box d2) ,z1 ] ,
       Delta = [z2],
       Seq = Gamma ==> Delta.

% proof
test_input(bob1, Seq,pdf) :-
       Gamma = [boxp p1, boxp ~d , diamc ~c, box a1, box ~d3 ,box diam b1, box diam b2 , box ( c v box d), g1],
       Delta = [d],
       Seq = Gamma ==> Delta.

% proof
test_input(bob2, Seq,pdf) :-
   Gamma= [boxp p1, diamc ~d1, diamc ~a1,  box a1, box diam ~c1, box diam ~c2,
	   box ( c1 v box d1),  box ( c2 v box d2), g1   ] ,
   Delta = [d2],
   Seq = Gamma ==> Delta.


% ref
test_input(bob3, Seq,pdf) :-
   Gamma = [boxp p1, diamc q1, box a1, box a2, box diam b1, box diam b2, box ( c v box d), g1 ],
   Delta = [g2],
   Seq = Gamma ==> Delta.

% ref
test_input(bob4, Seq,pdf) :-
   Gamma= [box diam a1, box diam a2, box ( b v box d)  ],
   Delta = [b],
   Seq = Gamma ==> Delta.

% ref
test_input(bob5, Seq,pdf) :-
  Gamma = [box diam p1, box diam p2, box ( ~ p1 v box p3) , p4 ],
  Delta = [p3],
  Seq = Gamma ==> Delta.

% proof
test_input(bob6, Seq,pdf) :-
  Gamma = [box diam b1,  box diam ~c1,box ( c1 v box ~b1)  ],
  Delta = [p3],
  Seq = Gamma ==> Delta.


% proof
test_input(bob7, Seq,pdf) :-
  Gamma = [box diam ~c1, box ( c1 v box d1), box (c2 v box ~d1)  ],
  Delta = [box c2],
  Seq = Gamma ==> Delta.


% proof
test_input(bob8, Seq,pdf) :-
   Gamma= [boxp p1, diamc ~d1, diamc ~a1,  box a1,  box diam ~c2,
	   box ( c1 v box d1),  box ( c2 v box d2), g1   ] ,
   Delta = [d2],
   Seq = Gamma ==> Delta.



%%   box_r

%proof
test_input(br1, Seq,pdf) :-
  Gamma = [box diam p2 ,  box ( ~ p1 v box ~p2)] ,
  Delta = [box ~p1],
  Seq = Gamma ==> Delta.


%ref
test_input(br2, Seq,pdf) :-
  Gamma = [box diam p1, box diam p2, box ( ~ p1 v box p3), p4],
  Delta = [box p3],
  Seq = Gamma ==> Delta.

%% box_diam_l
%

% ref
test_input(bod1, Seq,pdf) :-
  Gamma = [box diam ~p2 ,  box (p1 v diam p2)] ,
  Delta = [p1],
  Seq = Gamma ==> Delta.

% proof
test_input(bod2, Seq,pdf) :-
  Gamma = [box  ~p2 ,  box (p1 v diam p2)] ,
  Delta = [p1],
  Seq = Gamma ==> Delta.


% proof
test_input(bod3, Seq,pdf) :-
  Gamma = [boxp p1, box ~p2 ,  box (p1 v diam p2)] ,
  Delta = [p1],
  Seq = Gamma ==> Delta.



%%  PAPER EXAMPLES

%  Example 1 (ref)

test_input(example1, Seq,pdf) :-
	Gamma = [ diamc ~p4, box diam p1, box diam p2,   box diam ~p2,
		  box (~ p1 v box p3 ), p4 ],
	Delta = [ p3 ],
	Seq = Gamma ==> Delta.

%  Example 1 - a different refutation

test_input(example1bis, Seq,pdf) :-
	Gamma = [ box diam p1, box diam p2,   box diam ~p2,
		  box (~ p1 v box p3 ), p4 ],
	Delta = [box p4, p3],
	Seq = Gamma ==> Delta.


%  Example 2 (ref)

test_input(example2, Seq,pdf) :-
  E1 =  z1 <-->  (p1  & ~ p2 & ~ p3),
  E2 =  z2  <-->  (~ p1  &  p2 & ~ p3),
  E3 =  z3  <-->  (~ p1  & ~ p2 &  p3),
  Gamma = [ box  E1, box E2,  box E3,
	    box diam q, box diam ~q,
	    box ( ~ z2  v box ( ~ p1) ),  box ( ~ z3  v box p3 ) ,
	    box(~ z2 v diam z3 ) , box(~ z1 v diam z2 ) , diamc ~ ~ z1 ],
  Delta = [],
  Seq = Gamma ==> Delta.


%  Example 2 - a different refutation

test_input(example2bis, Seq,pdf) :-
  E1 =  z1 <-->  (p1  & ~ p2 & ~ p3),
  E2 =  z2  <-->  (~ p1  &  p2 & ~ p3),
  E3 =  z3  <-->  (~ p1  & ~ p2 &  p3),
  Gamma = [ box  E1, box E2,  box E3,
	    box diam q, box diam ~q,
	    box ( ~ z2  v box ( ~ p1) ),  box ( ~ z3  v box p3 ) ,
	    box(~ z2 v diam z3 ) , box(~ z1 v diam z2 )  ],
  Delta = [box ~z1],
  Seq = Gamma ==> Delta.



%%  branch [P.Abate, R.Goré: The Tableaux Work Bench, ENTCS, 2009]


%ref
test_input(branch2, Seq,pdf) :-
    Gamma = [q0 ,~q1 ,
	     box(   (q1 --> q0) & (q2 --> q1)  )  ,
	     box(  ~(q0 & p0)  v box(q0 --> p0)),
	     box(  ~(q0 & ~p0) v box(q0--> ~p0)),
	     box(  ~(q1 &  p1) v box(q1 --> p1)) ,
	     box(  ~(q1 & ~p1) v box(q1 --> ~p1)) ,
	     box(  ~(q0 & ~q1) v diam(q1 & ~q2 & ~p1)),
	     box(  ~(q0 & ~q1) v diam(q1 & ~q2 & p1))
	  ] ,
	Delta = [],
	Seq = Gamma ==> Delta.



%#########  MAIN  PREDICATES FOR TEST ############

% find_test_input(+Lab,-Seq,-Pdf)
%
% If a test with label Lab is defined, Seq and Pdf are instantiated
% as in test Lab.
% Otherwise, the computation is aborted
%
%


find_test_input(Lab,Seq,Pdf) :-
	test_input(Lab, Seq,Pdf) ->
	true
	;
	write('*** No tests are defined with label '),
	write(Lab), nl, abort.

%%	test(+Lab)
%
%  Run the  test with label Lab and print the result on the standard output
%  The Input of the test must be defined by the predicate  test_input/3


test(Lab) :-
	find_test_input(Lab, Seq,_) ->
	prove( Seq, Result ),
	writeln(Result).





%  test(+Lab, -Result: proof or ref)
%
%  Run the  test with label Lab and write the result in a latex file
%  The Input of the test must be defined by the predicate  test_input/3
%  The latex  file is placed in the directory defined test_dir/1

test(Lab, Result) :-
	test_dir(Test_dir),
	( exists_directory(Test_dir) ->  % create test_dir if it does not exist
	   true
            ;
           make_directory(Test_dir)
        ),
	find_test_input(Lab, Seq ,Pdf),
	write('*** INPUT TEST: '),
	writeln(Lab),
	prove( Seq, Result ),
	Seq = Gamma ==> _Delta,
	gammaBox(Gamma,GammaNoPrint),
  % formulas in GammaNoPrint are printed apart
	(
	is_proof(Result) ->
	   atomic_list_concat(['proof_', Lab, '.tex'],Tex_file),
	   proof_to_latex(Result,Test_dir,Tex_file,GammaNoPrint)
	;
	  atomic_list_concat(['ref_', Lab, '.tex'],Tex_file),
	  refutation_to_latex(Result,Test_dir, Tex_file, GammaNoPrint)
	),
	(   Pdf = pdf ->
	       generate_pdf(Test_dir,Tex_file)
	;
	      writeln('*** no pdf ')
	).

% run all the defined tests

alltests :-
	findall(Lab, test_input(Lab, _ , _), Lab_list),
	% Lab_list = { Lab | Lab is a label of a test input }
	list_to_set(Lab_list, Lab_set), %  remove duplicated labels
	findall( Lab, ( member(Lab,Lab_set) , test(Lab, _)) , _).



% gammaBox(+Gamma, -GammaBox)
%
% GammaBox = { box A | box A in Gamma and A is classic }

gammaBox([],[]).

gammaBox([ F  | Gamma ] , GammaBox ) :-
	gammaBox(Gamma, GammaBox1),
	(
	   ( F = box A ,  is_classical_formula(A)) ->
	GammaBox = [ F | GammaBox1]
	;
	GammaBox = GammaBox1
	).










































