%%   UTILITY
%
% Utility to print proofs and refutations

:- module(util, [
	       proof_to_latex/3, % +Proof, +Dir, +Latex_file
	       proof_to_latex/4, % +Proof, +Dir, +Latex_file , +GammaNoPrint
	       refutation_to_latex/3, % +Refutation, +Dir, +Latex_file,  +GammaNoPrint
	       refutation_to_latex/4, % +Refutation, +Dir, +Latex_file
	       generate_pdf/2	 % +Dir,+Latex_file
	      ]
	   ).

:-use_module(prover).


%%	RULE NAMES


write_rule('refCL') :- !, write('\\mathrm{refCL}').
write_rule('axCL') :-  !, write('\\mathrm{axCL}').
write_rule('refClu') :- !, write('\\mathrm{refClu}').
write_rule('axClu') :-  !, write('\\mathrm{axClu}').
write_rule('succ') :-  !,write('\\mathrm{succ}').
write_rule('succ_1') :-  !,write('\\ruleSucc_1').
write_rule('succ_2') :-  !,write('\\ruleSucc_2').
write_rule('box_box_l') :-  !,write('\\Box\\Diam L').
write_rule('box_diam_l') :-  !,write('\\Box\\Diam L').
write_rule('box_or_box_l') :-  !,write('\\BoxOrBoxL').
write_rule('box_or_box_l_1') :-  !,write('\\BoxOrBoxL_1').
write_rule('box_or_box_l_reg') :-  !,write('\\BoxOrBoxL_{reg}').
write_rule('box_or_box_l_spec') :-  !,write('\\BoxOrBoxL_{spec}').
write_rule('box_or_box_l_clu') :-  !,write('\\BoxOrBoxL_{clu}').
write_rule('box_or_box_l_2') :-  !,write('\\BoxOrBoxL_2').
write_rule('box_or_diam_l') :-  !,write('\\BoxOrDiamL').
write_rule('box_or_diam_l_1') :-  !,write('\\BoxOrDiamL_1').
write_rule('box_or_diam_l_2') :-  !,write('\\BoxOrDiamL_2').
write_rule('box_or_diam_l_reg') :-  !,write('\\BoxOrDiamL_{reg}').
write_rule('box_or_diam_l_spec') :-  !,write('\\BoxOrDiamL_{spec}').
write_rule('box_diam_l') :-  !,write('\\Box\\lor L').
write_rule('box_diam_l_1') :-  !,write('\\Box\\lor L_1').
write_rule('box_diam_l_2') :-  !,write('\\Box\\lor L_2').
write_rule('box_r') :-  !,write('\\Box R').
write_rule('box_r_1') :-  !,write('\\Box R_1').
write_rule('box_r_reg') :-  !,write('\\Box R_{reg}').
write_rule('box_r_spec') :-  !,write('\\Box R_{spec}').
write_rule('box_r_2') :-  !,write('\\Box R_2').

% Error: not defined R

write_rule(R) :- write('\\mbox{!!!! ' ), write(R), write('}').



%%%%%  	PRINT PROOFS AND REFUTATIONS
%
%
% proof_to_latex(+Proof: proof, +Dir , +Latex_file)
%
% Print the latex translation of Proof in Dir/Latex_file


proof_to_latex(Ref,Dir,Latex_file) :-
	proof_to_latex(Ref,Dir,Latex_file,[]).

% GammaNoPrint: Gamma formulas not to be printed

proof_to_latex(Proof,Dir,Latex_file,GammaNoPrint) :-
	atomic_list_concat([Dir, '/',Latex_file], File_path),
	tell(File_path), % open File_path
	write_preamble,
	writeln('\\begin{document}'),
	writeln('\\thispagestyle{empty}'),
	writeln('\\['),


	write_proof(Proof,GammaNoPrint),
	writeln('\\]'),
	write_gammaNoPrint(GammaNoPrint),
	writeln('\\end{document}'),
	told, % close open files
	write('*** '), write(File_path), writeln(' generated'),!.

proof_to_latex(_,_,_,_) :-  % errors
	told, % close open files
	writeln('***  proof_to_latex errors   ***'),
	abort.


% refutation_to_latex(+Refutation: ref,  +Dir , +Latex_file)
%
% Print the latex translation of Refutation in  Dir/Latex_file

refutation_to_latex(Ref,Dir,Latex_file) :-
	refutation_to_latex(Ref,Dir,Latex_file,[]).

% GammaNoPrint: Gamma formulas not to be printed

refutation_to_latex(Ref,Dir,Latex_file, GammaNoPrint) :-
	atomic_list_concat([Dir, '/',Latex_file], File_path),
	tell(File_path), % open File_path
	write_preamble,
	writeln('\\begin{document}'),
	writeln('\\thispagestyle{empty}'),
	writeln('\\['),
	write_ref(Ref,GammaNoPrint),
	writeln('\\]'),
	write_gammaNoPrint(GammaNoPrint),
	writeln('\\end{document}'),
	told, % close open files
	write('*** '), write(File_path), writeln(' generated'),!.

refutation_to_latex(_,_,_,_) :-  % errors
	told, % close open files
	writeln('***  refutation_to_latex errors   ***'),
	abort.

%  generate_pdf(+Dir,+Latex_file)
%
%  Generate pdf of Dir/Latex_file

generate_pdf(Dir,Latex_file) :-
	file_name_extension(Base_name, '.tex', Latex_file),
	% Base_name: base name of Latex_file
	atomic_list_concat([Dir, '/', Base_name], Base_name_path),
	atomic_list_concat([Dir, '/', Latex_file], Latex_file_path),
	atomic_list_concat(['pdflatex -interaction=batchmode  -output-directory ', Dir, ' ', Latex_file_path],
			   Pdflatex_cmd),
	write('*** '),
	writeln(Pdflatex_cmd),
	shell(Pdflatex_cmd,Status), % exec Pdflatex_cmd, Status is the outcome
	(
	Status == 0 ->
	  atomic_list_concat(['rm -f ', Base_name_path, '.aux ', Base_name_path, '.log '] , Clean_cmd),
	  write('*** '),
	  writeln(Clean_cmd),
	  shell(Clean_cmd),
	  write('***  '),
	 atomic_list_concat([Base_name_path,'.pdf'],Pdf_file_path),
	 write(Pdf_file_path),
	 writeln(' generated')
	; % pdflatex errors
	  told, % close open files
	  writeln('*** pdflatex errors'),
	  abort
	).


%##########   WRITE PROOFS ############

write_preamble:-
    writeln('\\documentclass[10pt]{article}'),
    writeln('\\usepackage{proof}'),
    writeln('\\usepackage{amssymb}'),
    %writeln('\\usepackage{tikz}'),
    %writeln('\\usepackage{color}'),
    writeln('\\usepackage{xifthen}'),
    %writeln('\\pdfpagewidth 80in'),
    writeln('\\pdfpagewidth 200in'),   % MAX  WIDTH
    writeln('%%\\pdfpageheight 100in'),
    nl,
    writeln('\\newcommand{\\arrowclu}{ \\stackrel{clu}{\\Rightarrow}}'),
    writeln('%%\\seq{Gamma}{Delta} writes the sequent Gamma ==> Delta'),
    writeln('\\newcommand{\\seq}[2]{'),
    writeln('\\ifthenelse{\\isempty{#1}}{\\emptyset}{#1}\\, \\Rightarrow\\,'),
    writeln('\\ifthenelse{\\isempty{#2}}{\\emptyset}{#2}'),
    writeln('}'),
    writeln('%%\\seqclu{Gamma}{Delta} writes the cluster sequent Gamma =clu=> Delta'),
    writeln('\\newcommand{\\seqclu}[2]{'),
    writeln('\\ifthenelse{\\isempty{#1}}{\\emptyset}{#1}\\, \\arrowclu\\,'),
    writeln('\\ifthenelse{\\isempty{#2}}{\\emptyset}{#2}'),
    writeln('}'),
    writeln('\\newcommand{\\Diam}{\\lozenge}'),
    writeln('\\newcommand{\\Diamc}{\\blacklozenge}'),
    writeln('\\newcommand{\\Boxp}{\\boxplus}'),
    writeln('\\newcommand{\\BoxOrBoxL}{\\Box\\!\\lor\\!\\Box L}'),
    writeln('\\newcommand{\\BoxOrDiamL}{\\Box\\!\\lor\\!\\Diamond L}'),
    writeln('\\newcommand{\\ruleSucc}{\\mathrm{Succ}}'),
    nl.




% write_formula(+F:formula)

is_unary_formula(F) :- is_atomic_formula(F).
is_unary_formula(~ _A ).
is_unary_formula(box _A ).
is_unary_formula(diam _A ).
is_unary_formula(boxp _A ).
is_unary_formula(diamc _A ).


write_formula(F):-
	is_atomic_formula(F), !,
	write(F).


write_formula(F):-
	is_unary_formula(F), !,   % F = ~A | box A | diam A | boxp A | diamc A
	F =.. [Op, A ],
	(
	 is_unary_formula(A) ->
	   write_op(Op),  write_formula(A)
	;
	  write_op(Op), write('('),   write_formula(A), write(')')
	).


write_formula(F):-  % F = A & B | A v B | A--> B |  A <--> B
	F =.. [Op, A , B], !,
	( is_unary_formula(A) ->
	    write_formula(A)
	 ;
	   write('('), write_formula(A), write(')')
	),
	write_op(Op),
	( is_unary_formula(B) ->
	    write_formula(B)
	 ;
	   write('('), write_formula(B), write(')')
	).

%%  write operators

write_op('~') :- !,write(' \\neg ').
write_op('&') :- !, write(' \\land ').
write_op('v') :- !, write(' \\lor ').
write_op('-->') :- !, write(' \\to ').
write_op('box') :- !,write(' \\Box ').
write_op('diam') :- !,write(' \\Diam ').
write_op('<-->') :- !, write('\\leftrightarrow ').
write_op('boxp') :- !,write(' \\Boxp ').
write_op('diamc') :- !,write(' \\Diamc ').
write_op(Op) :-  write(Op).

% write_formula_list(+L : list of formulas)

write_formula_list([]).
write_formula_list([A | L]) :-
	write_formula(A),
	(  L \= [] ->  	write(',\\,') ; true  ),
	write_formula_list(L).


% big_union( +L , -Gamma)
%
%  L: list of lists of formula
%  Gamma =  union of the lists in L

big_union([],[]).
big_union([ I] , I).
big_union([ I | L ], U) :-
	big_union(L, V),
	union(I,V,U).


% +write_gamma(Gamma,GammaNoPrint)
%
%  write the set Gamma; formulas in GammaNoPrint are primted apart

write_gamma(Gamma,GammaNoPrint) :-
	(
	 (    ground(GammaNoPrint) , GammaNoPrint = []) ->
	    write_formula_list(Gamma)
	;
	    subtract(Gamma,GammaNoPrint,GammaToPrint),
	    write('\\Gamma'),
	    (
	        GammaToPrint = [] ->
	          true
	             ;
	        write(',\\,')
	     ),
	    write_formula_list(GammaToPrint)
	).

% write_sequent(+Seq : sequent)

write_sequent( (GBox,GDiam,Gcl)  '=clu=>' Dcl, GammaNoPrint ) :- !,
	big_union( [ GBox,GDiam,Gcl],Gamma),
	write('\\seqclu{'),
	write_gamma(Gamma,GammaNoPrint),
	write('}{'),
	write_formula_list(Dcl),
	writeln('}').


write_sequent((ThBoxp,ThDiamc , GBox , GBoxDiam, GBoxOrBox, GBoxOrDiam ,Gcl) ==> (DBox, Dcl),
	     GammaNoPrint) :- !,
	big_union( [ ThBoxp,ThDiamc , GBox , GBoxDiam, GBoxOrBox, GBoxOrDiam ,Gcl] ,Gamma),
	big_union( [ DBox, Dcl], Delta),
	write('\\seq{'),
	write_gamma(Gamma,GammaNoPrint),
	write('}{'),
	write_formula_list(Delta),
	writeln('}').


write_gammaNoPrint(GammaNoPrint):-
	(
	GammaNoPrint=[] ->
	  true
	;
	writeln('\\['),
	writeln('\\Gamma\\;=\\;'),
	write_formula_list(GammaNoPrint),nl,
	writeln('\\]')
	).

%% write_proof(+Proof : proof)
%

write_proof(proof(Rule, Sequent, Premise_list),GammaNoPrint) :- !,
        write('\\infer['),
	write_rule(Rule),
	writeln(']{'),
	write_sequent(Sequent,GammaNoPrint),
	writeln('}{'),
	write_proof_premise_list(Premise_list, GammaNoPrint),
	writeln('}').


write_proof_premise_list([],_).

write_proof_premise_list([Proof], GammaNoPrint) :- !,
	write_proof(Proof,GammaNoPrint).

write_proof_premise_list([Proof | Proof_list], GammaNoPrint) :-
	write_proof(Proof, GammaNoPrint),
	write(' & '),
	write_proof_premise_list(Proof_list, GammaNoPrint).


%###########  WRITE REFUTATIONS AND  MODELS ####################

% write_ref(+Ref : ref)

write_ref(ref(Rule, Sequent, Ref_list), GammaNoPrint) :- !,
        write('\\infer['),
	write_rule(Rule),
	writeln(']{'),
	write_sequent(Sequent, GammaNoPrint),
	writeln('}{'),
	write_ref_list(Ref_list, GammaNoPrint),
	writeln('}').

write_ref_list([],_).

write_ref_list([Ref], GammaNoPrint) :-
	write_ref(Ref, GammaNoPrint).

write_ref_list([Ref | Ref_list], GammaNoPrint) :-
	write_ref(Ref,GammaNoPrint),
	write(' & '),
	write_ref_list(Ref_list, GammaNoPrint).


















